"use strict";
/*
 * Copyright 2019-2020 The Kale Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
///<reference path="../node_modules/@types/node/index.d.ts"/>
const application_1 = require("@jupyterlab/application");
const notebook_1 = require("@jupyterlab/notebook");
const docmanager_1 = require("@jupyterlab/docmanager");
const apputils_1 = require("@jupyterlab/apputils");
const coreutils_1 = require("@phosphor/coreutils");
const React = require("react");
require("../style/index.css");
const LeftPanelWidget_1 = require("./widgets/LeftPanelWidget");
const NotebookUtils_1 = require("./lib/NotebookUtils");
const RPCUtils_1 = require("./lib/RPCUtils");
/* tslint:disable */
exports.IKubeflowKale = new coreutils_1.Token('kubeflow-kale:IKubeflowKale');
const id = 'kubeflow-kale:deploymentPanel';
/**
 * Adds a visual Kubeflow Pipelines Deployment tool to the sidebar.
 */
exports.default = {
    activate,
    id,
    requires: [application_1.ILabShell, application_1.ILayoutRestorer, notebook_1.INotebookTracker, docmanager_1.IDocumentManager],
    provides: exports.IKubeflowKale,
    autoStart: true,
};
function activate(lab, labShell, restorer, tracker, docManager) {
    return __awaiter(this, void 0, void 0, function* () {
        let widget;
        const kernel = yield NotebookUtils_1.default.createNewKernel();
        window.addEventListener('beforeunload', () => kernel.shutdown());
        window.addEventListener('unhandledrejection', RPCUtils_1.globalUnhandledRejection);
        // TODO: backend can become an Enum that indicates the type of
        //  env we are in (like Local Laptop, MiniKF, GCP, UI without Kale, ...)
        const backend = yield getBackend(kernel);
        let rokError = null;
        if (backend) {
            try {
                yield RPCUtils_1.executeRpc(kernel, 'log.setup_logging');
            }
            catch (error) {
                RPCUtils_1.globalUnhandledRejection({ reason: error });
                throw error;
            }
            try {
                yield RPCUtils_1.executeRpc(kernel, 'rok.check_rok_availability');
            }
            catch (error) {
                const unexpectedErrorCodes = [
                    RPCUtils_1.RPC_CALL_STATUS.EncodingError,
                    RPCUtils_1.RPC_CALL_STATUS.ImportError,
                    RPCUtils_1.RPC_CALL_STATUS.UnhandledError,
                ];
                if (error instanceof RPCUtils_1.RPCError &&
                    !unexpectedErrorCodes.includes(error.error.code)) {
                    rokError = error.error;
                    console.warn('Rok is not available', rokError);
                }
                else {
                    RPCUtils_1.globalUnhandledRejection({ reason: error });
                    throw error;
                }
            }
        }
        else {
            rokError = {
                rpc: 'rok.check_rok_availability',
                code: RPCUtils_1.RPC_CALL_STATUS.ImportError,
                err_message: 'Rok is not available',
                err_details: 'To use this Rok feature you first need Kale running' +
                    ' in the backend.',
                err_cls: 'importError',
            };
            console.warn('Rok is not available', rokError);
        }
        /**
         * Detect if Kale is installed
         */
        function getBackend(kernel) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    yield NotebookUtils_1.default.sendKernelRequest(kernel, `import kale`, {});
                }
                catch (error) {
                    console.error('Kale backend is not installed.');
                    return false;
                }
                return true;
            });
        }
        function loadPanel() {
            return __awaiter(this, void 0, void 0, function* () {
                let reveal_widget = undefined;
                if (backend) {
                    // Check if NOTEBOOK_PATH env variable exists and if so load
                    // that Notebook
                    const path = yield RPCUtils_1.executeRpc(kernel, 'nb.resume_notebook_path');
                    if (path) {
                        console.log('Resuming notebook ' + path);
                        // open the notebook panel
                        reveal_widget = docManager.openOrReveal(path);
                    }
                }
                // add widget
                if (!widget.isAttached) {
                    labShell.add(widget, 'left');
                }
                // open widget if resuming from a notebook
                if (reveal_widget) {
                    // open kale panel
                    widget.activate();
                }
            });
        }
        // Creates the left side bar widget once the app has fully started
        lab.started.then(() => {
            // show list of commands in the commandRegistry
            // console.log(lab.commands.listCommands());
            widget = apputils_1.ReactWidget.create(React.createElement(LeftPanelWidget_1.KubeflowKaleLeftPanel, { lab: lab, tracker: tracker, docManager: docManager, backend: backend, kernel: kernel, rokError: rokError }));
            widget.id = 'kubeflow-kale/kubeflowDeployment';
            widget.title.iconClass = 'jp-kale-logo jp-SideBar-tabIcon';
            widget.title.caption = 'Kubeflow Pipelines Deployment Panel';
            restorer.add(widget, widget.id);
        });
        // Initialize once the application shell has been restored
        // and all the widgets have been added to the NotebookTracker
        lab.restored.then(() => {
            loadPanel();
        });
        return { widget };
    });
}
